use sea_orm::prelude::*;
use teloxide::{adaptors::Throttle, prelude::*};

use crate::{errors::HandlerUsed, PinnedFuture};

pub async fn generic<F>(
    bot: Throttle<Bot>,
    text: String,
    msg: Message,
    db: DatabaseConnection,
    dialogue: crate::handlers::MainDialogue,
    check: F,
    next: crate::handlers::PackagedHandler<String>,
) -> crate::Result<()>
where
    for<'a> F: FnOnce(
        &'a Throttle<Bot>,
        &'a Message,
        &'a DatabaseConnection,
        &'a str,
    ) -> PinnedFuture<'a, crate::Result<bool>>,
{
    if text == "/cancel" {
        dialogue.exit().await?;
        bot.send_message(msg.chat.id, "Successfully cancelled")
            .await?;
        return Ok(());
    }
    match check(&bot, &msg, &db, &text).await {
        Ok(true) => (),
        Ok(false) => {
            dialogue.exit().await?;
            return Ok(());
        }
        Err(err) => {
            let _ = dialogue.exit().await;
            return Err(err);
        }
    };
    next.lock().await.take().ok_or(HandlerUsed)?(bot, msg, db, dialogue, text).await
}
