use super::{MainDialogue, PackagedHandler};
use sea_orm::DatabaseConnection;
use std::{future::Future, sync::Arc};
use teloxide::{adaptors::Throttle, prelude::*};
use tokio::sync::Mutex;

#[inline]
pub fn package_handler<H, F, T>(f: H) -> PackagedHandler<T>
where
    H: FnOnce(Throttle<Bot>, Message, DatabaseConnection, MainDialogue, T) -> F
        + Send
        + Sync
        + 'static,
    F: Future<Output = crate::Result<()>> + Send + 'static,
{
    Arc::new(Mutex::new(Some(Box::new(
        move |bot, msg, db, dialogue, val| Box::pin(f(bot, msg, db, dialogue, val)),
    ))))
}

pub async fn delete_message(bot: Throttle<Bot>, msg: Message) {
    let _ = bot.delete_message(msg.chat.id, msg.id).await;
}
